@@ -1,6 +1,6 @@
 package Dancer2::Plugin::Auth::Extensible;
 
-our $VERSION = '0.611';
+our $VERSION = '0.612';
 
 use strict;
 use warnings;
@@ -159,7 +159,8 @@ has realm_providers => (
 # hooks
 #
 
-plugin_hooks qw(login_required permission_denied after_login_success);
+plugin_hooks qw(before_authenticate_user login_required permission_denied
+  after_login_success);
 
 #
 # keywords
@@ -301,6 +302,9 @@ sub auth_provider {
 sub authenticate_user {
     my ( $plugin, $username, $password, $realm ) = @_;
 
+    $plugin->execute_plugin_hook( 'before_authenticate_user',
+        { username => $username, password => $password, realm => $realm } );
+
     my @realms_to_check = $realm ? ($realm) : ( keys %{ $plugin->realms } );
 
     for my $realm (@realms_to_check) {
@@ -1246,6 +1250,10 @@ and should do at least the following:
             params->{username}, params->{password}
         );
         if ($success) {
+            # change session ID if we have a new enough D2 version with support
+            # (security best practice on privilege level change)
+            app->change_session_id
+                if app->can('change_session_id');
             session logged_in_user => params->{username};
             session logged_in_user_realm => $realm;
             # other code here
@@ -1735,6 +1743,12 @@ specified by that realm's config.
 
 This plugin provides the following hooks:
 
+=head2 before_authenticate_user
+
+Called at the start of L</authenticate_user>.
+
+Receives a hash reference of username, password and realm.
+
 =head2 login_required
 
 =head2 permission_denied
@@ -1781,7 +1795,7 @@ Documentation fix by Vince Willems.
 
 Henk van Oers (GH #8, #13).
 
-Andrew Beverly (GH #6, #7, #10, #17, #22, #24, #25, #26).
+Andrew Beverly (GH #6, #7, #10, #17, #22, #24, #25, #26, #54).
 This includes support for creating and editing users and manage user passwords.
 
 Gabor Szabo (GH #11, #16, #18).
